
Plane = Core.class()

function Plane:init(width, height)
  self:resize(width, height)
end

-- resize quads (overwrites any previous data)
function Plane:resize(width, height)
  self.width = width
  self.height = height
  self:reset()
  self.plane = nil
end

-- creates empty quads
function Plane:reset()
  local width, height = self.width, self.height
  local quads = {}
  for q = 1, 4 do
    local quad = {}
    for r = 1, width * height do
      quad[#quad + 1] = 0 -- generate empty cell
    end
    quads[#quads + 1] = quad -- store quad
  end
  self.quads = quads
end

-- returns the desired quad
function Plane:getQuad(q)
  return self.quads[q]
end

-- returns the entire plane
function Plane:getPlane()
  local plane = self.plane
  
  if plane == nil then
    
    local quads, width, height = self.quads, self.width, self.height
    
    plane = {}
    
    local q1, q2 = quads[1], quads[2]
    for r = 0, height - 1 do
      for c = 1, width do
        plane[#plane + 1] = q1[(r * width) + c]
      end
      for c = 1, width do
        plane[#plane + 1] = q2[(r * width) + c]
      end
    end

    q1, q2 = quads[3], quads[4]
    for r = 0, height - 1 do
      for c = 1, width do
        plane[#plane + 1] = q1[(r * width) + c]
      end
      for c = 1, width do
        plane[#plane + 1] = q2[(r * width) + c]
      end
    end
  end
  
  return plane
end

-- returns the x, y position of an empty cell in this quad
function Plane:getPosition(q, d)
  local floor, random = math.floor, math.random
  
  d = d or 1 -- if no data supplied then 1 = occupied
  
  local width = self.width
  
  local quad = self.quads[q]
  local cells = {}
  
  for i = 1, #quad do -- generate a list of empty cells
    if quad[i] == 0 then -- cell is empty so can accomodate an object
      cells[#cells + 1] = i -- add to list of empty cells
    end
  end
  
  local c = cells[random(1, #cells)] -- choose empty cell at random
  quad[c] = d -- mark cell occupied

  local x = (c - 1) % width
  return x + 1 , floor((c - x) / width) + 1-- return x, y position
end
